import cockpit from "cockpit";
import React from "react";
import { ClipboardCopy } from "@patternfly/react-core";

const _ = cockpit.gettext;

export function getConfigData (sshAddress = null) {
    return [
        {
            id: "systemd-timesyncd",
            title: _("Set clock from NTP-Server"),
            type: "config",
            helpText: (
                <div className='help-text'>
                    <p>{_("This setting synchronizes your system time with an NTP server for accuracy.")}</p>
                    <p><strong>{_("Why is this needed?")}</strong></p>
                    <ul>
                        <li>{_("Ensures accurate timestamps for logs and scheduled tasks.")}</li>
                        <li>{_("Prevents time drift on devices without a real-time clock.")}</li>
                    </ul>
                    <p>
                        <strong>{_("Requirements")}:</strong> {_("An active internet connection is required to sync with NTP servers.")}
                    </p>
                </div>
            )
        },
        {
            id: "ssh",
            title: _("SSH daemon to enable remote login"),
            type: "config",
            helpText: (
                <div className='help-text'>
                    <p>{_("Enabling SSH allows remote access to this device over a network.")}</p>
                    <p><strong>{_("Security Considerations")}:</strong></p>
                    <ul>
                        <li>{_("Use strong passwords or SSH key-based authentication.")}</li>
                        <li>{_("Disable SSH if remote access is not required.")}</li>
                    </ul>
                    <p><strong>{_("Usage")}:</strong> {_("To connect via SSH, use")}: </p>
                    <ClipboardCopy
                        isReadOnly
                        style={{ marginTop: "0.5rem" }}
                    >
                        ssh {sshAddress}
                    </ClipboardCopy>
                </div>
            )
        },
        {
            id: "gui",
            title: _("Boot to GUI"),
            type: "config",
            rebootRequired: true,
            helpText: (
                <div className='help-text'>
                    <p>{_("When enabled, the system boots into a graphical user interface (GUI) instead of the command line.")}</p>
                    <p><strong>{_("Considerations")}:</strong></p>
                    <ul>
                        <li>{_("GUI mode uses more RAM and processing power.")}</li>
                        <li>{_("Recommended for users unfamiliar with command-line operations.")}</li>
                    </ul>
                </div>
            )
        },
        {
            id: "var-log.mount",
            title: _("Store logs on"),
            type: "config",
            rebootRequired: true,
            options: {
                enabled: {
                    title: _("Temporary storage")
                },
                disabled: {
                    title: _("Flash storage")
                }
            },
            helpText: (
                <div className='help-text'>
                    <p>{_("Configures where system logs are stored.")}</p>
                    <div>
                        <strong>{_("Storage Options")}:</strong>
                        <div>
                            <p>
                                <strong>{_("Temporary storage")}:</strong>{" "}
                                {_(
                                    "Logs are stored in RAM and lost on reboot. Reduces flash wear but makes debugging harder."
                                )}
                            </p>
                            <p>
                                <strong>{_("Flash storage")}:</strong>{" "}
                                {_(
                                    "Logs are stored persistently. Useful for debugging."
                                )}
                            </p>
                        </div>
                    </div>
                </div>
            )
        },
        {
            id: "bluetooth",
            title: _("BT"),
            type: "config",
            rebootRequired: true,
            helpText: (
                <div className='help-text'>
                    <p>{_("Enables or disables the Bluetooth module.")}</p>
                    <p><strong>{_("Usage")}:</strong></p>
                    <ul>
                        <li>{_("Enable if using Bluetooth peripherals such as keyboards or audio devices.")}</li>
                        <li>{_("Disable to reduce power consumption and improve security.")}</li>
                    </ul>
                </div>
            )
        },
        {
            id: "ieee80211",
            title: _("WLAN"),
            type: "config",
            rebootRequired: true,
            helpText: (
                <div className='help-text'>
                    <p>{_("Controls the built-in WiFi module.")}</p>
                    <p><strong>{_("Notes")}:</strong></p>
                    <ul>
                        <li>{_("Ensure correct country code settings to prevent connection issues.")}</li>
                        <li>{_("Disabling WLAN can reduce power consumption and increase security.")}</li>
                    </ul>
                </div>
            )
        },
        {
            id: "external-antenna",
            title: _("Select antenna"),
            type: "config",
            rebootRequired: true,
            options: {
                enabled: {
                    title: _("External antenna (SMA)")
                },
                disabled: {
                    title: _("Internal antenna")
                }
            },
            helpText: (
                <div className='help-text'>
                    <p>{_("Switches between an internal and an external antenna for wireless connectivity.")}</p>
                    <div>
                        <p><strong>{_("Antenna Selection")}:</strong></p>
                        <ul>
                            <li>
                                <strong>{_("External antenna (SMA)")}:</strong>{" "}
                                {_("Provides better signal reception in challenging environments.")}
                            </li>
                            <li>
                                <strong>{_("Internal antenna")}:</strong>{" "}
                                {_("Uses the built-in antenna, reducing external hardware needs.")}
                            </li>
                        </ul>
                    </div>
                </div>

            )
        },
        {
            id: "nodered",
            title: _("Node-RED"),
            type: "service",
            helpText: (
                <div className='help-text'>
                    <p>
                        {_(
                            "Node-RED is a flow-based programming tool for connecting hardware, APIs, and online services. Its drag-and-drop interface allows quick setup, testing, and automation of processes. For RevPi users, it simplifies configuration and experimentation before deployment."
                        )}
                    </p>
                    <p><strong>{_("Common Uses")}:</strong></p>
                    <ul>
                        <li>{_("Automating IoT workflows.")}</li>
                        <li>{_("Creating dashboards for real-time monitoring.")}</li>
                    </ul>
                    <p>
                        <strong>{_("Access")}:</strong>{" "}
                        {_("Once enabled, access Node-RED via the Node-RED link above.")}
                    </p>
                </div>

            )
        },
        {
            id: "noderedrevpinodes-server",
            title: _("Node-RED RevPi Nodes Server"),
            type: "service",
            helpText: (
                <div className='help-text'>
                    <p>{_("Enables additional functionality for Node-RED by integrating RevPi-specific nodes.")}</p>
                    <p><strong>{_("Benefits")}:</strong></p>
                    <ul>
                        <li>{_("Allows control and monitoring of RevPi hardware directly from Node-RED.")}</li>
                        <li>{_("Preinstalled and optimized for RevPi devices.")}</li>
                    </ul>
                    <p><strong>{_("Important")}:</strong></p>
                    {_("Necessary to use the RevPi Node-RED nodes!")}
                </div>
            )
        },
        {
            id: "revpipyload",
            title: "RevPiPyLoad",
            type: "service",
            helpText: (
                <div className='help-text'>
                    <p>{_("RevPiPyLoad is a Python-based runtime environment for RevPi.")}</p>
                    <p><strong>{_("Functions")}:</strong></p>
                    <ul>
                        <li>{_("Manages real-time industrial automation tasks.")}</li>
                        <li>{_("Runs user-defined Python scripts.")}</li>
                    </ul>
                    <p>{_("Ensure your scripts are compatible with RevPiPyLoad before enabling.")}</p>
                </div>
            )
        },
        {
            id: "avahi",
            title: _("Avahi (ZeroConf)"),
            type: "service",
            helpText: (
                <div className='help-text'>
                    <p>{_("Avahi enables ZeroConf networking, allowing easy discovery of network services.")}</p>
                    <p><strong>{_("Examples of Use")}:</strong></p>
                    <ul>
                        <li>{_("Accessing the device using `hostname.local` instead of an IP address.")}</li>
                        <li>{_("Automatically discovering printers and file shares.")}</li>
                    </ul>
                </div>
            )
        },
        {
            id: "dphys-swapfile",
            title: _("Swap / page file (recommended for GUI)"),
            type: "service",
            helpText: (
                <div className='help-text'>
                    <p>{_("Enables swap memory, which acts as virtual RAM when physical memory is low.")}</p>
                    <p><strong>{_("Impact")}:</strong></p>
                    <ul>
                        <li>{_("Prevents system crashes when RAM is exhausted.")}</li>
                        <li>{_("Can slow performance since swap is stored on disk.")}</li>
                    </ul>
                </div>
            )
        }
    ];
}
