import cockpit from "cockpit";
import { getConfigData } from "./RevPiConfigData.jsx";
import { useLoggedInUser } from "../../../pkg/lib/hooks.js";
import { useEffect } from "react";

let _instance = null;

export class RevPiConfigService {
    static getInstance () {
        _instance ??= new RevPiConfigService();
        return _instance;
    }

    constructor () {
        cockpit.event_target(this);
        this.configData = getConfigData();
        this.user = null; // Initially null until explicitly initialized
        this.userInitialized = false; // Flag to ensure init() runs only once
    }

    /** Explicitly initialize the service with user data (only runs once) */
    initUser (user) {
        if (!this.userInitialized) {
            this.user = user;
            this.userInitialized = true;
            this.configData = getConfigData(this.sshAddress);
        } else {
            console.warn("RevPiConfigService has already been initialized. Ignoring duplicate init call.");
        }
    }

    refresh () {
        this.dispatchEvent("changed");
    }

    async callRevpiConfig (command, ids = []) {
        return await cockpit.spawn(["/usr/bin/revpi-config", command, ...ids], { superuser: "require" });
    }

    async toggleConfiguration (id, isEnabled) {
        const command = isEnabled ? "disable" : "enable";
        return this.callRevpiConfig(command, [id]);
    }

    async hasLowMemory () {
        const minimumMemory = 1536;
        try {
            const output = await cockpit.spawn(["free", "-m"]);
            const match = output.match(/^Mem:\s+(\d+)/m);

            if (match) {
                const ramMb = parseInt(match[1], 10);
                return ramMb <= minimumMemory; // Return true if 1536 MB or less, false otherwise
            }

            console.error("Could not determine RAM size.");
            return false; // Default to false if parsing fails
        } catch (error) {
            console.error("Error checking RAM size:", error);
            return false; // Return false in case of error
        }
    }

    getStatus (id, response) {
        if (response.length === 1) {
            switch (response[0]) {
            case "0":
                return "disabled";
            case "1":
                return "enabled";
            case "2":
                return "unavailable";
            default:
                console.log(`Error: Unexpected response: ${response}. Response must be either 0, 1, or 2.`);
            }
        }
        return response[0];
    }

    parseResponseToObject (responseString) {
        // Initialize an array to hold the parsed response objects
        const parsedResponse = [];

        // Split the response string into an array of individual response items
        const responseArray = responseString.trim()
            .split(" ");

        // Store the lengths of the response and the configData
        const nResponses = responseArray.length;
        const nConfigData = this.configData.length;

        // use a counter to keep track of the multi responses per item
        let indexShifts = 0;
        for (let i = 0; i < this.configData.length; i++) {
            // Retrieve the current configuration item
            const item = this.configData[i];

            // Determine the number of response items expected for this configuration item
            const responseItems = item.responseCount ? item.responseCount : 1;

            // Calculate the start and end index for slicing the response array
            const startIndex = i + indexShifts;
            const endIndex = i + indexShifts + responseItems;

            const rawStatus = responseArray.slice(startIndex, endIndex);
            const status = this.getStatus(item.id, rawStatus);
            const responseObject = {
                ...item,
                status,
                isEnabled: status === "enabled",
                isAvailable: status !== "unavailable",
                hasError: false,
                isLoading: false
            };
            parsedResponse.push(responseObject);

            // Shift the index in case more than one response item has been used
            indexShifts += responseItems - 1;
        }

        // Check if the total number of response items matches the number expected based on configuration
        if (nResponses - indexShifts !== nConfigData) {
            throw new Error("Error: An issue occurred with the index shifting logic.");
        }
        return parsedResponse;
    }

    async getRevpiConfigStatus () {
        const ids = this.configData.map((item) => item.id);
        const response = await this.callRevpiConfig("availstat", ids);
        return this.parseResponseToObject(response);
    }

    get hostName () {
        return cockpit.transport.host === "localhost" ? window.location.hostname : cockpit.transport.host.split("@").at(-1);
    }

    get sshAddress () {
        return `${this.user?.name}@${this.hostName}`;
    }
}

// write a hook to load user (if more hooks are required, better to rewrite the class into a functional component
export function useRevPiConfigService () {
    const user = useLoggedInUser(); // Get the logged-in user
    const service = RevPiConfigService.getInstance(); // Just get the singleton

    useEffect(() => {
        if (user) {
            service.initUser(user); // Initialize only once
        }
    }, [user, service]);

    return service;
}
